from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class LightCapabilitiesStreaming(HueObject):
    @property
    def renderer(self) -> bool:
        return map_from_dict(self._raw, "capabilities", "streaming", "renderer")

    @property
    def proxy(self) -> bool:
        return map_from_dict(self._raw, "capabilities", "streaming", "proxy")


class LightCapabilitiesControlCt(HueObject):
    @property
    def min(self) -> int:
        return map_from_dict(self._raw, "capabilities", "control", "ct", "min")

    @property
    def max(self) -> int:
        return map_from_dict(self._raw, "capabilities", "control", "ct", "max")


class LightCapabilitiesControl(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.ct = LightCapabilitiesControlCt("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.ct.load_data(self._raw)

    @property
    def mindimlevel(self) -> int:
        return map_from_dict(self._raw, "capabilities", "control", "mindimlevel")

    @property
    def maxlumen(self) -> int:
        return map_from_dict(self._raw, "capabilities", "control", "maxlumen")

    @property
    def colorgamuttype(self) -> str:
        return map_from_dict(self._raw, "capabilities", "control", "colorgamuttype")

    @property
    def colorgamut(self) -> list[float, float, float]:
        return map_from_dict(self._raw, "capabilities", "control", "colorgamut")


class LightCapabilities(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.streaming = LightCapabilitiesStreaming("", uri, raw=raw)
        self.control = LightCapabilitiesControl("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.streaming.load_data(self._raw)
        self.control.load_data(self._raw)

    @property
    def certified(self) -> bool:
        return map_from_dict(self._raw, "capabilities", "certified")


class LightState(HueObject):
    @property
    def on(self) -> object:
        return map_from_dict(self._raw, "state", "on")

    @on.setter
    def on(self, on: bool) -> None:
        self.set(on=on)

    @property
    def bri(self) -> int:
        return map_from_dict(self._raw, "state", "bri")

    @bri.setter
    def bri(self, bri: int) -> None:
        self.set(bri=bri)

    @property
    def hue(self) -> int:
        return map_from_dict(self._raw, "state", "hue")

    @hue.setter
    def hue(self, hue: int) -> None:
        self.set(hue=hue)

    @property
    def sat(self) -> int:
        return map_from_dict(self._raw, "state", "sat")

    @sat.setter
    def sat(self, sat: int) -> None:
        self.set(sat=sat)

    @property
    def xy(self) -> list[float, float]:
        return map_from_dict(self._raw, "state", "xy")

    @xy.setter
    def xy(self, xy: list[float, float]) -> None:
        self.set(xy=xy)

    @property
    def ct(self) -> int:
        return map_from_dict(self._raw, "state", "ct")

    @ct.setter
    def ct(self, ct: int) -> None:
        self.set(ct=ct)

    @property
    def alert(self) -> str:
        return map_from_dict(self._raw, "state", "alert")

    @alert.setter
    def alert(self, alert: str) -> None:
        self.set(alert=alert)

    @property
    def effect(self) -> str:
        return map_from_dict(self._raw, "state", "effect")

    @effect.setter
    def effect(self, effect: str) -> None:
        self.set(effect=effect)

    @property
    def colormode(self) -> str:
        return map_from_dict(self._raw, "state", "colormode")

    @colormode.setter
    def colormode(self, colormode: str) -> None:
        self.set(colormode=colormode)

    @property
    def reachable(self) -> bool:
        return map_from_dict(self._raw, "state", "reachable")

    @reachable.setter
    def reachable(self, reachable: bool) -> None:
        self.set(reachable=reachable)

    @property
    def transitiontime(self) -> None:
        return None

    @transitiontime.setter
    def transitiontime(self, transitiontime: int) -> None:
        self.set(transitiontime=transitiontime)

    @property
    def bri_inc(self) -> None:
        return None

    @bri_inc.setter
    def bri_inc(self, bri_inc: int) -> None:
        self.set(bri_inc=bri_inc)

    @property
    def hue_inc(self) -> None:
        return None

    @hue_inc.setter
    def hue_inc(self, hue_inc: int) -> None:
        self.set(hue_inc=hue_inc)

    @property
    def sat_inc(self) -> None:
        return None

    @sat_inc.setter
    def sat_inc(self, sat_inc: int) -> None:
        self.set(sat_inc=sat_inc)

    @property
    def xy_inc(self) -> None:
        return None

    @xy_inc.setter
    def xy_inc(self, xy_inc: list[float, float]) -> None:
        self.set(xy_inc=xy_inc)

    @property
    def ct_inc(self) -> None:
        return None

    @ct_inc.setter
    def ct_inc(self, ct_inc: int) -> None:
        self.set(ct_inc=ct_inc)

    def set(self, on: bool = None, bri: int = None, hue: int = None, sat: int = None, xy: list[float, float] = None,
            ct: int = None, alert: str = None, effect: str = None, colormode: str = None,
            reachable: bool = None, transitiontime: int = None, bri_inc: int = None, hue_inc: int = None,
            sat_inc: int = None, xy_inc: list[float, float] = None, ct_inc: int = None) -> None:
        val = {}
        if on is not None:
            val["on"] = on
        if bri is not None:
            val["bri"] = bri
        if hue is not None:
            val["hue"] = hue
        if sat is not None:
            val["sat"] = sat
        if xy is not None:
            val["xy"] = xy
        if ct is not None:
            val["ct"] = ct
        if alert is not None:
            val["alert"] = alert
        if effect is not None:
            val["effect"] = effect
        if colormode is not None:
            val["colormode"] = colormode
        if reachable is not None:
            val["reachable"] = reachable
        if transitiontime is not None:
            val["transitiontime"] = transitiontime
        if bri_inc is not None:
            val["bri_inc"] = bri_inc
        if hue_inc is not None:
            val["hue_inc"] = hue_inc
        if sat_inc is not None:
            val["sat_inc"] = sat_inc
        if xy_inc is not None:
            val["xy_inc"] = xy_inc
        if ct_inc is not None:
            val["ct_inc"] = ct_inc
        self.set_data("state", val)


class Light(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.state = LightState("", uri, raw=raw)
        self.capabilities = LightCapabilities("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.state.load_data(self._raw)
        self.capabilities.load_data(self._raw)

    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def type(self) -> str:
        return map_from_dict(self._raw, "type")

    @property
    def modelid(self) -> str:
        return map_from_dict(self._raw, "modelid")

    @property
    def uniqueid(self) -> str:
        return map_from_dict(self._raw, "uniqueid")

    @property
    def manufacturername(self) -> str:
        return map_from_dict(self._raw, "manufacturername")

    @property
    def productname(self) -> str:
        return map_from_dict(self._raw, "productname")

    @property
    def luminaireuniqueid(self) -> str:
        return map_from_dict(self._raw, "luminaireuniqueid")

    @property
    def swversion(self) -> str:
        return map_from_dict(self._raw, "swversion")

    def switch_on(self) -> None:
        self.state.on = True

    def switch_off(self) -> None:
        self.state.on = False

    def toggle(self) -> None:
        if self.state.on:
            self.switch_off()
        else:
            self.switch_on()

    def set(self, name: str = None) -> None:
        val = {}
        if name is not None:
            val["name"] = name
        self.set_data("", val)



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class ResourceLinks(HueObject):
    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def description(self) -> str:
        return map_from_dict(self._raw, "description")

    @description.setter
    def description(self, description: str) -> None:
        self.set(description=description)

    @property
    def resource_type(self) -> str:
        return map_from_dict(self._raw, "type")

    @resource_type.setter
    def resource_type(self, resource_type: str) -> None:
        self.set(resource_type=resource_type)

    @property
    def classid(self) -> int:
        return map_from_dict(self._raw, "classid")

    @classid.setter
    def classid(self, classid: int) -> None:
        self.set(classid=classid)

    @property
    def owner(self) -> str:
        return map_from_dict(self._raw, "owner")

    @owner.setter
    def owner(self, owner: str) -> None:
        self.set(owner=owner)

    @property
    def recycle(self) -> bool:
        return map_from_dict(self._raw, "recycle")

    @recycle.setter
    def recycle(self, recycle: bool) -> None:
        self.set(recycle=recycle)

    @property
    def links(self) -> list[str]:
        return map_from_dict(self._raw, "links")

    @links.setter
    def links(self, links: list[str]) -> None:
        self.set(links=links)

    def set(self, name: str = None, description: str = None, resource_type: str = None, classid: int = None,
            owner: str = None, recycle: bool = None, links: list[str] = None) -> None:
        val = {}
        if name is not None:
            val["name"] = name
        if description is not None:
            val["description"] = description
        if resource_type is not None:
            val["type"] = type
        if classid is not None:
            val["classid"] = classid
        if owner is not None:
            val["owner"] = owner
        if recycle is not None:
            val["recycle"] = recycle
        if links is not None:
            val["links"] = links
        self.set_data("", val)



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class Backup(HueObject):
    @property
    def status(self) -> str:
        return map_from_dict(self._raw, "backup", "status")

    @property
    def errorcode(self) -> int:
        return map_from_dict(self._raw, "backup", "errorcode")


class InternetServices(HueObject):
    @property
    def internet(self) -> str:
        return map_from_dict(self._raw, "internetservices", "internet")

    @property
    def remoteaccess(self) -> str:
        return map_from_dict(self._raw, "internetservices", "remoteaccess")

    @property
    def time(self) -> str:
        return map_from_dict(self._raw, "internetservices", "time")

    @property
    def swupdate(self) -> str:
        return map_from_dict(self._raw, "internetservices", "swupdate")


class AutoInstall(HueObject):
    @property
    def on(self) -> bool:
        return map_from_dict(self._raw, "swupdate2", "autoinstall", "on")

    @property
    def updatetime(self) -> str:
        return map_from_dict(self._raw, "swupdate2", "autoinstall", "updatetime")


class DeviceTypes(HueObject):
    @property
    def bridge(self) -> bool:
        return map_from_dict(self._raw, "swupdate", "devicetypes", "bridge")

    @property
    def lights(self) -> list:
        return map_from_dict(self._raw, "swupdate", "devicetypes", "lights")

    @property
    def sensors(self) -> list:
        return map_from_dict(self._raw, "swupdate", "devicetypes", "sensors")


class SWUpdate(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.devicetypes = DeviceTypes("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.devicetypes.load_data(self._raw)

    @property
    def checkforupdate(self) -> bool:
        return map_from_dict(self._raw, "swupdate", "checkforupdate")

    @checkforupdate.setter
    def checkforupdate(self, checkforupdate: bool) -> None:
        self.set(checkforupdate=checkforupdate)

    @property
    def updatestate(self) -> int:
        return map_from_dict(self._raw, "swupdate", "updatestate")

    @property
    def notify(self) -> bool:
        return map_from_dict(self._raw, "swupdate", "notify")

    @notify.setter
    def notify(self, notify: bool) -> None:
        self.set(notify=notify)

    @property
    def url(self) -> str:
        return map_from_dict(self._raw, "swupdate", "url")

    @url.setter
    def url(self, url: str) -> None:
        self.set(url=url)

    @property
    def text(self) -> str:
        return map_from_dict(self._raw, "swupdate", "text")

    @text.setter
    def text(self, text: str) -> None:
        self.set(text=text)

    def set(self, checkforupdate: bool = None, notify: bool = None, url: str = None, text: str = None) -> None:
        val = {"swupdate": {}}
        if checkforupdate is not None:
            val["swupdate"]["checkforupdate"] = checkforupdate
        if notify is not None:
            val["swupdate"]["notify"] = notify
        if url is not None:
            val["swupdate"]["url"] = url
        if text is not None:
            val["swupdate"]["text"] = text
        self.set_data("", val)


class SWUpdate2(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.autoinstall = AutoInstall("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.autoinstall.load_data(self._raw)

    @property
    def bridge(self) -> dict:
        return map_from_dict(self._raw, "swupdate2", "bridge")

    @property
    def checkforupdate(self) -> bool:
        return map_from_dict(self._raw, "swupdate2", "checkforupdate")

    @checkforupdate.setter
    def checkforupdate(self, checkforupdate: bool) -> None:
        self.set(checkforupdate=checkforupdate)

    @property
    def state(self) -> str:
        return map_from_dict(self._raw, "swupdate2", "state")

    @property
    def install(self) -> bool:
        return map_from_dict(self._raw, "swupdate2", "install")

    @install.setter
    def install(self, install: bool) -> None:
        self.set(install=install)

    @property
    def lastchange(self) -> str:
        return map_from_dict(self._raw, "swupdate2", "lastchange")

    @property
    def lastinstall(self) -> str:
        return map_from_dict(self._raw, "swupdate2", "lastinstall")

    def set(self, checkforupdate: bool = None, install: bool = None) -> None:
        val = {"swupdate2": {}}
        if checkforupdate is not None:
            val["swupdate2"]["checkforupdate"] = checkforupdate
        if install is not None:
            val["swupdate2"]["install"] = install
        self.set_data("", val)


class Configuration(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.swupdate = SWUpdate("", uri, raw=raw)
        self.swupdate2 = SWUpdate2("", uri, raw=raw)
        self.internetservices = InternetServices("", uri, raw=raw)
        self.backup = Backup("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.swupdate.load_data(self._raw)
        self.swupdate2.load_data(self._raw)
        self.internetservices.load_data(self._raw)
        self.backup.load_data(self._raw)

    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def whitelist(self) -> dict:
        return map_from_dict(self._raw, "whitelist")

    @property
    def portalstate(self) -> dict:
        return map_from_dict(self._raw, "portalstate")

    @property
    def apiversion(self) -> str:
        return map_from_dict(self._raw, "apiversion")

    @property
    def swversion(self) -> str:
        return map_from_dict(self._raw, "swversion")

    @property
    def proxyaddress(self) -> str:
        return map_from_dict(self._raw, "proxyaddress")

    @proxyaddress.setter
    def proxyaddress(self, proxyaddress: str) -> None:
        self.set(proxyaddress=proxyaddress)

    @property
    def proxyport(self) -> int:
        return map_from_dict(self._raw, "proxyport")

    @proxyport.setter
    def proxyport(self, proxyport: int) -> None:
        self.set(proxyport=proxyport)

    @property
    def linkbutton(self) -> bool:
        return map_from_dict(self._raw, "linkbutton")

    @linkbutton.setter
    def linkbutton(self, linkbutton: bool) -> None:
        self.set(linkbutton=linkbutton)

    @property
    def ipaddress(self) -> str:
        return map_from_dict(self._raw, "ipaddress")

    @ipaddress.setter
    def ipaddress(self, ipaddress: str) -> None:
        self.set(ipaddress=ipaddress)

    @property
    def mac(self) -> str:
        return map_from_dict(self._raw, "mac")

    @property
    def netmask(self) -> str:
        return map_from_dict(self._raw, "netmask")

    @netmask.setter
    def netmask(self, netmask: str) -> None:
        self.set(netmask=netmask)

    @property
    def gateway(self) -> str:
        return map_from_dict(self._raw, "gateway")

    @gateway.setter
    def gateway(self, gateway: str) -> None:
        self.set(gateway=gateway)

    @property
    def dhcp(self) -> bool:
        return map_from_dict(self._raw, "dhcp")

    @dhcp.setter
    def dhcp(self, dhcp: bool) -> None:
        self.set(dhcp=dhcp)

    @property
    def portalservices(self) -> bool:
        return map_from_dict(self._raw, "portalservices")

    @property
    def utc(self) -> str:
        return map_from_dict(self._raw, "UTC")

    @utc.setter
    def utc(self, utc: str) -> None:
        self.set(utc=utc)

    @property
    def localtime(self) -> str:
        return map_from_dict(self._raw, "localtime")

    @property
    def timezone(self) -> str:
        return map_from_dict(self._raw, "timezone")

    @timezone.setter
    def timezone(self, timezone: str) -> None:
        self.set(timezone=timezone)

    @property
    def zigbeechannel(self) -> int:
        return map_from_dict(self._raw, "zigbeechannel")

    @zigbeechannel.setter
    def zigbeechannel(self, zigbeechannel: int) -> None:
        self.set(zigbeechannel=zigbeechannel)

    @property
    def modelid(self) -> str:
        return map_from_dict(self._raw, "modelid")

    @property
    def bridgeid(self) -> str:
        return map_from_dict(self._raw, "bridgeid")

    @property
    def factorynew(self) -> bool:
        return map_from_dict(self._raw, "factorynew")

    @property
    def replacesbridgeid(self) -> str:
        return map_from_dict(self._raw, "replacesbridgeid")

    @property
    def datastoreversion(self) -> str:
        return map_from_dict(self._raw, "datastoreversion")

    @property
    def starterkitid(self) -> str:
        return map_from_dict(self._raw, "starterkitid")

    @property
    def touchlink(self) -> bool:
        return map_from_dict(self._raw, "touchlink")

    @touchlink.setter
    def touchlink(self, touchlink: bool) -> None:
        self.set(touchlink=touchlink)

    def set(self, name: str = None, proxyaddress: str = None, proxyport: int = None, linkbutton: bool = None,
            ipaddress: str = None, netmask: str = None, gateway: str = None, dhcp: bool = None, utc: str = None,
            timezone: str = None, zigbeechannel: int = None, touchlink: bool = None) -> None:
        val = {}
        if name is not None:
            val["name"] = name
        if proxyaddress is not None:
            val["proxyaddress"] = proxyaddress
        if proxyport is not None:
            val["proxyport"] = proxyport
        if linkbutton is not None:
            val["linkbutton"] = linkbutton
        if ipaddress is not None:
            val["ipaddress"] = ipaddress
        if netmask is not None:
            val["netmask"] = netmask
        if gateway is not None:
            val["gateway"] = gateway
        if dhcp is not None:
            val["dhcp"] = dhcp
        if utc is not None:
            val["UTC"] = utc
        if timezone is not None:
            val["timezone"] = timezone
        if zigbeechannel is not None:
            val["zigbeechannel"] = zigbeechannel
        if touchlink is not None:
            val["touchlink"] = touchlink
        self.set_data("", val)



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class RuleAction:
    def __init__(self, action: dict) -> None:
        self.address = map_from_dict(action, "address")
        self.method = map_from_dict(action, "method")
        self.body = map_from_dict(action, "body")

    def as_dict(self) -> dict:
        a = {}
        if self.address is not None:
            a["address"] = self.address
        if self.method is not None:
            a["method"] = self.method
        if self.body is not None:
            a["body"] = self.body
        return a


class RuleCondition:
    def __init__(self, condition: dict) -> None:
        self.address = map_from_dict(condition, "address")
        self.operator = map_from_dict(condition, "operator")
        self.value = map_from_dict(condition, "value")

    def as_dict(self) -> dict:
        d = {}
        if self.address is not None:
            d["address"] = self.address
        if self.operator is not None:
            d["operator"] = self.operator
        if self.value is not None:
            d["value"] = self.value
        return d


class Rule(HueObject):
    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def owner(self) -> str:
        return map_from_dict(self._raw, "owner")

    @property
    def created(self) -> str:
        return map_from_dict(self._raw, "created")

    @property
    def lasttriggered(self) -> str:
        return map_from_dict(self._raw, "lasttriggered")

    @property
    def timestriggered(self) -> str:
        return map_from_dict(self._raw, "timestriggered")

    @property
    def status(self) -> str:
        return map_from_dict(self._raw, "status")

    @status.setter
    def status(self, status: str) -> None:
        self.set(status=status)

    @property
    def conditions(self) -> list[RuleCondition]:
        return [RuleCondition(condition) for condition in map_from_dict(self._raw, "conditions")]

    @conditions.setter
    def conditions(self, conditions: list[RuleCondition]) -> None:
        condition_list = []
        for condition in conditions:
            condition_list.append(condition.as_dict())
        self.set(conditions=condition_list)

    @property
    def actions(self) -> list[RuleAction]:
        return [RuleAction(action) for action in map_from_dict(self._raw, "actions")]

    @actions.setter
    def actions(self, actions: list[RuleAction]) -> None:
        action_list = []
        for action in actions:
            action_list.append(action.as_dict())
        self.set(actions=action_list)

    def set(self, name: str = None, status: str = None, conditions: list[dict] = None, actions: list[dict] = None):
        val = {}
        if name is not None:
            val["name"] = name
        if status is not None:
            val["status"] = status
        if conditions is not None:
            val["conditions"] = conditions
        if actions is not None:
            val["actions"] = actions
        self.set_data("", val)



import requests
import json


# get item from dictionary multiple levels deep
def map_from_dict(dictionary: dict, *items: str, default: object = None):
    d = dictionary
    for item in items:
        if item not in d:
            return default
        elif item == items[-1]:
            return d[item]
        elif type(d[item]) == dict:
            d = d[item]
        else:
            return default
    return default


# scan for new hue-devices using provided uri
def scan_new(scan_uri: str) -> list:
    new_items = []
    req = requests.get(scan_uri)
    if req.status_code == 200:
        for item in req.json():
            if item != "lastscan":
                new_items.append(item)
    return new_items


class HueObject:
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self._identity = identity
        self._uri = uri
        self._raw = None
        self._response_status_code = 0
        self._response_json = None
        self._response_message = ""
        if raw is None:
            self.load_data()
        else:
            self.load_data(raw)

    @property
    def identity(self):
        return self._identity

    # parse response from requests.response object and return whether REST call was successful
    # in case of successful API call, if ID is included in the response, _response_message holds the ID
    def parse_response(self, response: requests.Response) -> bool:
        self._response_status_code = response.status_code
        self._response_json = response.json()
        if self._response_status_code == 200:
            if isinstance(self._response_json, list) and "success" in self._response_json[0] and \
                    "id" in self._response_json[0]["success"]:
                self._response_message = self._response_json[0]["success"]["id"]
            else:
                self._response_message = ""
            return True
        else:
            if isinstance(self._response_json, list) and len(self._response_json) > 0 and \
                    "error" in self._response_json[0] and "description" in self._response_json[0]["error"]:
                self._response_message = self._response_json[0]["error"]["description"]
            else:
                self._response_message = "Unknown error"
            return False

    # get data from REST API. If provided, directly assign to prevent needing to make API call
    def load_data(self, raw: dict = None) -> None:
        if raw is not None:
            self._raw = raw
        else:
            if not self.parse_response(requests.get(self._uri)):
                raise Exception(self._response_message)
            else:
                self._raw = self._response_json

    # set data through REST call; optionally reload data to sync status with actual bridge status
    def set_data(self, category: str, data: dict, reload: bool = True) -> None:
        if not self.parse_response(requests.put(self._uri + "/" + category, data=json.dumps(data))):
            raise Exception(self._response_message)
        elif reload:
            self.load_data()

    # delete object through REST call; after successful delete object is invalid (state reset to None)
    def delete(self) -> None:
        if not self.parse_response(requests.delete(self._uri)):
            self._raw = None
            raise Exception(self._response_message)

    # pretty print object
    def __str__(self):
        return "ID        : " + self._identity + "\n" + \
               "URI       : " + self._uri + "\n" + \
               "Data      : \n" + \
               "None" if self._raw is None else json.dumps(self._raw, sort_keys=False, indent=4)

    # getter for items not explicitly available in subclass
    def get_item(self, *args: str) -> object:
        return map_from_dict(self._raw, *args)

    # setter for items not explicitly available in subclass
    def set_item(self, category: str, item: str, value: object) -> None:
        self.set_data(category, {item: value})



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class SceneAppData(HueObject):
    @property
    def version(self) -> int:
        return map_from_dict(self._raw, "appdata", "version")

    @property
    def data(self) -> str:
        return map_from_dict(self._raw, "appdata", "data")


class SceneLightState:
    def __init__(self, state: dict = None, on: bool = None, bri: int = None, hue: int = None, sat: int = None,
                 xy: list[float, float] = None, ct: int = None, effect: str = None, transitiontime: int = None) -> None:
        if state is not None:
            self.on = map_from_dict(state, "on")
            self.bri = map_from_dict(state, "bri")
            self.hue = map_from_dict(state, "hue")
            self.sat = map_from_dict(state, "sat")
            self.xy = map_from_dict(state, "xy")
            self.ct = map_from_dict(state, "ct")
            self.effect = map_from_dict(state, "effect")
            self.transitiontime = map_from_dict(state, "transitiontime")
        else:
            self.on = on
            self.bri = bri
            self.hue = hue
            self.sat = sat
            self.xy = xy
            self.ct = ct
            self.effect = effect
            self.transitiontime = transitiontime

    def as_dict(self) -> dict:
        result = {}
        if self.on is not None:
            result["on"] = self.on
        if self.bri is not None:
            result["bri"] = self.bri
        if self.hue is not None:
            result["hue"] = self.hue
        if self.sat is not None:
            result["sat"] = self.sat
        if self.xy is not None:
            result["xy"] = self.xy
        if self.ct is not None:
            result["ct"] = self.ct
        if self.effect is not None:
            result["effect"] = self.effect
        if self.transitiontime is not None:
            result["transitiontime"] = self.transitiontime
        return result


class SceneLightStateList(HueObject):
    def __getitem__(self, key: str) -> SceneLightState:
        return SceneLightState(state=map_from_dict(self._raw, "lightstates", key))

    def __setitem__(self, key: str, value: SceneLightState) -> None:
        self.set_data("lightstates/" + key, value.as_dict())


class Scene(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.appdata = SceneAppData("", uri, raw=raw)
        self.lightstates = SceneLightStateList("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw=None) -> None:
        super().load_data(raw)
        self.appdata.load_data(self._raw)
        self.lightstates.load_data(self._raw)

    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def type(self) -> str:
        return map_from_dict(self._raw, "type")

    @property
    def group(self) -> str:
        return map_from_dict(self._raw, "group")

    @property
    def lights(self) -> list[str]:
        return map_from_dict(self._raw, "lights")

    @lights.setter
    def lights(self, lights: list[str]) -> None:
        self.set(lights=lights)

    @property
    def owner(self) -> str:
        return map_from_dict(self._raw, "owner")

    @property
    def recycle(self) -> bool:
        return map_from_dict(self._raw, "recycle")

    @property
    def locked(self) -> bool:
        return map_from_dict(self._raw, "locked")

    @property
    def picture(self) -> str:
        return map_from_dict(self._raw, "picture")

    @property
    def image(self) -> str:
        return map_from_dict(self._raw, "image")

    @property
    def lastupdated(self) -> str:
        return map_from_dict(self._raw, "lastupdated")

    @property
    def version(self) -> int:
        return map_from_dict(self._raw, "version")

    def set(self, name: str = None, lights: list[str] = None) -> None:
        val = {}
        if name is not None:
            val["name"] = name
        if lights is not None:
            val["lights"] = lights
        self.set_data("", val)



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class ScheduleCommand(HueObject):
    @property
    def address(self) -> str:
        return map_from_dict(self._raw, "command", "address")

    @address.setter
    def address(self, address: str) -> None:
        self.set(address=address)

    @property
    def method(self) -> str:
        return map_from_dict(self._raw, "command", "method")

    @method.setter
    def method(self, method: str) -> None:
        self.set(method=method)

    @property
    def body(self) -> str:
        return map_from_dict(self._raw, "command", "body")

    @body.setter
    def body(self, body: str) -> None:
        self.set(body=body)

    def set(self, address: str = None, method: str = None, body: str = None) -> None:
        val = {"command": {}}
        if address is not None:
            val["command"]["address"] = address
        if method is not None:
            val["command"]["method"] = method
        if body is not None:
            val["command"]["body"] = body
        self.set_data("", val)


class Schedule(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.command = ScheduleCommand("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.command.load_data(self._raw)

    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def description(self) -> str:
        return map_from_dict(self._raw, "description")

    @description.setter
    def description(self, description: str) -> None:
        self.set(description=description)

    @property
    def localtime(self) -> str:
        return map_from_dict(self._raw, "localtime")

    @localtime.setter
    def localtime(self, localtime: str) -> None:
        self.set(localtime=localtime)

    @property
    def starttime(self) -> str:
        return map_from_dict(self._raw, "starttime")

    @starttime.setter
    def starttime(self, starttime: str) -> None:
        self.set(starttime=starttime)

    @property
    def status(self) -> str:
        return map_from_dict(self._raw, "status")

    @status.setter
    def status(self, status: str) -> None:
        self.set(status=status)

    @property
    def autodelete(self) -> bool:
        return map_from_dict(self._raw, "autodelete")

    @autodelete.setter
    def autodelete(self, autodelete: bool) -> None:
        self.set(autodelete=autodelete)

    def set(self, name: str = None, description: str = None, localtime: str = None, starttime: str = None,
            status: str = None, autodelete: bool = None) -> None:
        val = {}
        if name is not None:
            val["name"] = name
        if description is not None:
            val["description"] = description
        if localtime is not None:
            val["localtime"] = localtime
        if starttime is not None:
            val["starttime"] = starttime
        if status is not None:
            val["status"] = status
        if autodelete is not None:
            val["autodelete"] = autodelete
        self.set_data("", val)



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class SensorState(HueObject):
    @property
    def presence(self) -> bool:
        return map_from_dict(self._raw, "state", "presence")

    @presence.setter
    def presence(self, presence: bool) -> None:
        self.set(presence=presence)

    @property
    def daylight(self) -> bool:
        return map_from_dict(self._raw, "state", "daylight")

    @property
    def buttonevent(self) -> int:
        return map_from_dict(self._raw, "state", "buttonevent")

    @property
    def lastupdated(self) -> str:
        return map_from_dict(self._raw, "state", "lastupdated")

    def set(self, presence: bool = None) -> None:
        val = {}
        if presence is not None:
            val["presence"] = presence
        self.set_data("state", val)


class SensorConfig(HueObject):
    @property
    def on(self) -> bool:
        return map_from_dict(self._raw, "config", "on")

    @on.setter
    def on(self, on: bool) -> None:
        self.set(on=on)

    @property
    def reachable(self) -> bool:
        return map_from_dict(self._raw, "config", "reachable")

    @property
    def battery(self) -> int:
        return map_from_dict(self._raw, "config", "battery")

    def set(self, on: bool = None) -> None:
        val = {}
        if on is not None:
            val["on"] = on
        self.set_data("config", val)


class Sensor(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.config = SensorConfig("", uri, raw=raw)
        self.state = SensorState("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.config.load_data(self._raw)
        self.state.load_data(self._raw)

    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def type(self) -> str:
        return map_from_dict(self._raw, "type")

    @property
    def modelid(self) -> str:
        return map_from_dict(self._raw, "modelid")

    @property
    def uniqueid(self) -> str:
        return map_from_dict(self._raw, "uniqueid")

    @property
    def manufacturername(self) -> str:
        return map_from_dict(self._raw, "manufacturername")

    @property
    def swversion(self) -> str:
        return map_from_dict(self._raw, "swversion")

    @property
    def recycle(self) -> bool:
        return map_from_dict(self._raw, "recycle")

    def set(self, name: str = None) -> None:
        val = {}
        if name is not None:
            val["name"] = name
        self.set_data("", val)



from setuptools import setup


setup(
    name='PythonHueControl',
    version='0.3.6.1',
    packages=['pythonhuecontrol', 'pythonhuecontrol.v1', 'pythonhuecontrol.v1.configuration',
              'pythonhuecontrol.v1.rule', 'pythonhuecontrol.v1.group',
              'pythonhuecontrol.v1.light', 'pythonhuecontrol.v1.scene', 'pythonhuecontrol.v1.bridge',
              'pythonhuecontrol.v1.sensor', 'pythonhuecontrol.v1.schedule', 'pythonhuecontrol.v1.resourcelinks',
              'pythonhuecontrol.v2'],
    package_dir={'': '.'},
    keywords='Hue Python API Philips Signify',
    url='https://github.com/elnkosc/PythonHueControl',
    license='GPL',
    author='Koen',
    author_email='koen@schilders.org',
    description='Feature Rich Python API for Hue V1 and V2',
    long_description=open('pythonhuecontrol/README.md', 'rt').read(),
    long_description_content_type='text/markdown'
)



import requests
import time
import json
from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict
from pythonhuecontrol.v1 import scan_new
from pythonhuecontrol.v1.configuration import Configuration
from pythonhuecontrol.v1.light import Light
from pythonhuecontrol.v1.group import Group
from pythonhuecontrol.v1.sensor import Sensor
from pythonhuecontrol.v1.rule import Rule
from pythonhuecontrol.v1.scene import Scene
from pythonhuecontrol.v1.schedule import Schedule
from pythonhuecontrol.v1.resourcelinks import ResourceLinks


def discover_bridge() -> str:
    ipaddress = None
    req = requests.get("https://discovery.meethue.com/")
    json_data = req.json()
    if req.status_code == 200 and len(json_data) > 0 and "internalipaddress" in json_data[0]:
        ipaddress = json_data[0]["internalipaddress"]
    return ipaddress


def create_bridge_user(uri: str, device_type: str, generate_client_key: bool = False,
                       wait_time: int = 60) -> tuple[str, str]:
    seconds = 0
    username = None
    clientkey = None
    while seconds < wait_time and username is None:
        time.sleep(1)
        seconds += 1
        json_data = {"devicetype": device_type}
        if generate_client_key:
            json_data["generateclientkey"] = True
        req = requests.post(uri, data=json.dumps(json_data))
        if req.status_code == 200:
            username = map_from_dict(req.json()[0], "success", "username")
            clientkey = map_from_dict(req.json()[0], "success", "clientkey")
    return username, clientkey


class Bridge(HueObject):
    def __init__(self, identity: str, uri: str, clientkey: str = None) -> None:
        self.registration_uri = uri
        self.config_uri = uri + "/config"
        self.groups_uri = uri + "/groups"
        self.lights_uri = uri + "/lights"
        self.sensors_uri = uri + "/sensors"
        self.schedules_uri = uri + "/schedules"
        self.scenes_uri = uri + "/scenes"
        self.rules_uri = uri + "/rules"
        self.resourcelinks_uri = uri + "/resourcelinks"
        self.new_lights_uri = uri + "/lights/new"
        self.new_sensors_uri = uri + "/sensors/new"
        self.clientkey = clientkey
        super().__init__(identity, uri)

    @property
    def light_ids(self) -> dict:
        return map_from_dict(self._raw, "lights")

    @property
    def group_ids(self) -> dict:
        return map_from_dict(self._raw, "groups")

    @property
    def sensor_ids(self) -> dict:
        return map_from_dict(self._raw, "sensors")

    @property
    def scene_ids(self) -> dict:
        return map_from_dict(self._raw, "scenes")

    @property
    def rule_ids(self) -> dict:
        return map_from_dict(self._raw, "rules")

    @property
    def schedule_ids(self) -> dict:
        return map_from_dict(self._raw, "schedules")

    @property
    def resourcelinks_ids(self) -> dict:
        return map_from_dict(self._raw, "resourcelinks")

    @property
    def config(self) -> Configuration:
        return Configuration("", self.config_uri, raw=map_from_dict(self._raw, "config"))

    def light(self, light_id: str) -> Light:
        return Light(light_id, self.lights_uri + "/" + light_id,
                     raw=map_from_dict(self._raw, "lights", light_id))

    def group(self, group_id: str) -> Group:
        return Group(group_id, self.groups_uri + "/" + group_id,
                     raw=map_from_dict(self._raw, "groups", group_id))

    def sensor(self, sensor_id: str) -> Sensor:
        return Sensor(sensor_id, self.sensors_uri + "/" + sensor_id,
                      raw=map_from_dict(self._raw, "sensors", sensor_id))

    def schedule(self, schedule_id: str) -> Schedule:
        return Schedule(schedule_id, self.schedules_uri + "/" + schedule_id,
                        raw=map_from_dict(self._raw, "schedules", schedule_id))

    def scene(self, scene_id: str) -> Scene:
        return Scene(scene_id, self.scenes_uri + "/" + scene_id,
                     raw=map_from_dict(self._raw, "scenes", scene_id))

    def rule(self, rule_id: str) -> Rule:
        return Rule(rule_id, self.rules_uri + "/" + rule_id,
                    raw=map_from_dict(self._raw, "rules", rule_id))

    def resourcelinks(self, resourcelink_id: str) -> ResourceLinks:
        return ResourceLinks(resourcelink_id, self.resourcelinks_uri + "/" + resourcelink_id,
                             raw=map_from_dict(self._raw, "resourcelinks", resourcelink_id))

    def new_sensors(self) -> list:
        return scan_new(self.new_sensors_uri)

    def new_lights(self) -> list:
        return scan_new(self.new_lights_uri)

    def search_lights(self) -> None:
        requests.post(self.lights_uri, data={})
        self.load_data()

    def search_sensors(self) -> None:
        requests.post(self.sensors_uri, data={})
        self.load_data()

    def create_group(self, name: str, lights: dict, group_type: str = "LightGroup",
                     group_class: str = "Other") -> Group:
        json_data = {"name": name, "type": group_type, "lights": lights}

        # item class only present for Rooms
        if group_type == "Room":
            json_data["class"] = group_class
        if self.parse_response(requests.post(self.groups_uri, data=json.dumps(json_data))):
            if self._response_json == "":
                raise Exception("Unknown Error: Group ID unavailable")
            return self.group(self._response_message)

    def create_schedule(self, name: str, localtime: str, address: str, method: str, body: dict,
                        description: str = "", status: str = "enabled", autodelete: bool = True,
                        recycle: bool = False) -> Schedule:
        json_data = {"name": name, "localtime": localtime,
                     "command": {"address": address, "method": method, "body": body},
                     "description": description, "status": status, "autodelete": autodelete, "recycle": recycle}
        if self.parse_response(requests.post(self.schedules_uri, data=json.dumps(json_data))):
            if self._response_message == "":
                raise Exception("Unknown Error: Schedule ID unavailable")
            return self.schedule(self._response_message)

    def create_rule(self, name: str, status: str, conditions: list, actions: list) -> Rule:
        json_data = {"name": name, "status": status, "conditions": conditions, "actions": actions}
        if self.parse_response(requests.post(self.rules_uri, data=json.dumps(json_data))):
            if self._response_message == "":
                raise Exception("Unknown Error: Rule ID unavailable")
            return self.rule(self._response_message)

    def create_scene(self, name: str, recycle: bool, scene_type: str, lights: list = None, group: str = None) -> Scene:
        if lights is None and group is None:
            raise Exception("Either lights or group should be specified")

        if group is not None:
            json_data = {"name": name, "recycle": recycle, "type": scene_type, "group": group}
        else:
            json_data = {"name": name, "recycle": recycle, "type": scene_type, "lights": lights}

        if self.parse_response(requests.post(self.scenes_uri, data=json.dumps(json_data))):
            if self._response_message == "":
                raise Exception("Unknown Error: Scene ID unavailable")
            return self.scene(self._response_message)

    def create_lightstates_scene(self, name: str, lights: list, appdata: dict, lightstates: dict) -> Scene:
        json_data = {"name": name, "lights": lights, "appdata": appdata, "lightstates": lightstates}
        if self.parse_response(requests.post(self.scenes_uri, data=json.dumps(json_data))):
            if self._response_message == "":
                raise Exception("Unknown Error: Scene ID unavailable")
            return self.scene(self._response_message)

    def create_resourcelinks(self, name: str, description: str = None, resource_type: str = None, classid: int = None,
                             owner: str = None, recycle: bool = None, links: list[str] = None) -> ResourceLinks:
        json_data = {"name": name}
        if description is not None:
            json_data["description"] = description
        if resource_type is not None:
            json_data["type"] = type
        if classid is not None:
            json_data["classid"] = classid
        if owner is not None:
            json_data["owner"] = owner
        if recycle is not None:
            json_data["recycle"] = recycle
        if links is not None:
            json_data["links"] = links
        if self.parse_response(requests.post(self.resourcelinks_uri, data=json.dumps(json_data))):
            if self._response_message == "":
                raise Exception("Unknown Error: Resourcelinks ID unavailable")
            return self.resourcelinks(self._response_message)



from pythonhuecontrol.v1 import HueObject
from pythonhuecontrol.v1 import map_from_dict


class GroupState(HueObject):
    @property
    def all_on(self) -> bool:
        return map_from_dict(self._raw, "state", "all_on")

    @property
    def any_on(self) -> bool:
        return map_from_dict(self._raw, "state", "any_on")


class GroupAction(HueObject):
    @property
    def on(self) -> bool:
        return map_from_dict(self._raw, "action", "on")

    @on.setter
    def on(self, on: bool) -> None:
        self.set(on=on)

    @property
    def bri(self) -> int:
        return map_from_dict(self._raw, "action", "bri")

    @bri.setter
    def bri(self, bri: int) -> None:
        self.set(bri=bri)

    @property
    def hue(self) -> int:
        return map_from_dict(self._raw, "action", "hue")

    @hue.setter
    def hue(self, hue: int) -> None:
        self.set(hue=hue)

    @property
    def sat(self) -> int:
        return map_from_dict(self._raw, "action", "sat")

    @sat.setter
    def sat(self, sat: int) -> None:
        self.set(sat=sat)

    @property
    def xy(self) -> list[float, float]:
        return map_from_dict(self._raw, "action", "xy")

    @xy.setter
    def xy(self, xy: list[float, float]) -> None:
        self.set(xy=xy)

    @property
    def ct(self) -> int:
        return map_from_dict(self._raw, "action", "ct")

    @ct.setter
    def ct(self, ct: int) -> None:
        self.set(ct=ct)

    @property
    def alert(self) -> str:
        return map_from_dict(self._raw, "action", "alert")

    @alert.setter
    def alert(self, alert: str) -> None:
        self.set(alert=alert)

    @property
    def effect(self) -> str:
        return map_from_dict(self._raw, "action", "effect")

    @effect.setter
    def effect(self, effect: str) -> None:
        self.set(effect=effect)

    @property
    def transitiontime(self) -> int:
        return map_from_dict(self._raw, "action", "transitiontime")

    @transitiontime.setter
    def transitiontime(self, transitiontime: int) -> None:
        self.set(transitiontime=transitiontime)

    @property
    def scene(self) -> str:
        return map_from_dict(self._raw, "action", "scene")

    @scene.setter
    def scene(self, scene: str) -> None:
        self.set(scene=scene)

    @property
    def bri_inc(self) -> int:
        return map_from_dict(self._raw, "action", "bri_inc")

    @bri_inc.setter
    def bri_inc(self, bri_inc: int) -> None:
        self.set(bri_inc=bri_inc)

    @property
    def hue_inc(self) -> int:
        return map_from_dict(self._raw, "action", "hue_inc")

    @hue_inc.setter
    def hue_inc(self, hue_inc: int) -> None:
        self.set(hue_inc=hue_inc)

    @property
    def sat_inc(self) -> int:
        return map_from_dict(self._raw, "action", "sat")

    @sat_inc.setter
    def sat_inc(self, sat_inc: int) -> None:
        self.set(sat_inc=sat_inc)

    @property
    def xy_inc(self) -> list[float, float]:
        return map_from_dict(self._raw, "action", "xy_inc")

    @xy_inc.setter
    def xy_inc(self, xy_inc: list[float, float]) -> None:
        self.set(xy_inc=xy_inc)

    @property
    def ct_inc(self) -> int:
        return map_from_dict(self._raw, "action", "ct_inc")

    @ct_inc.setter
    def ct_inc(self, ct_inc: int) -> None:
        self.set(ct_inc=ct_inc)

    def set(self, on: bool = None, bri: int = None, hue: int = None, sat: int = None, xy: list[float, float] = None,
            ct: int = None, alert: str = None, effect: str = None, transitiontime: int = None, scene: str = None,
            bri_inc: int = None, hue_inc: int = None, sat_inc: int = None, xy_inc: list[float, float] = None,
            ct_inc: int = None) -> None:
        val = {}
        if on is not None:
            val["on"] = on
        if bri is not None:
            val["bri"] = bri
        if hue is not None:
            val["hue"] = hue
        if sat is not None:
            val["sat"] = sat
        if xy is not None:
            val["xy"] = xy
        if ct is not None:
            val["ct"] = ct
        if alert is not None:
            val["alert"] = alert
        if effect is not None:
            val["effect"] = effect
        if transitiontime is not None:
            val["transitiontime"] = transitiontime
        if scene is not None:
            val["scene"] = scene
        if bri_inc is not None:
            val["bri_inc"] = bri_inc
        if hue_inc is not None:
            val["hue_inc"] = hue_inc
        if sat_inc is not None:
            val["sat_inc"] = sat_inc
        if xy_inc is not None:
            val["xy_inc"] = xy_inc
        if ct_inc is not None:
            val["ct_inc"] = ct_inc
        self.set_data("action", val)


class GroupPresence(HueObject):
    @property
    def lastupdated(self) -> str:
        return map_from_dict(self._raw, "presence", "lastupdated")

    @property
    def presence(self) -> bool:
        return map_from_dict(self._raw, "presence", "presence")

    @property
    def presence_all(self) -> bool:
        return map_from_dict(self._raw, "presence", "presence_all")


class GroupLightLevel(HueObject):
    @property
    def state(self) -> dict:
        return map_from_dict(self._raw, "lightlevel", "state")

    @property
    def lastupdated(self) -> str:
        return map_from_dict(self._raw, "lightlevel", "lastupdated")

    @property
    def dark(self) -> bool:
        return map_from_dict(self._raw, "lightlevel", "dark")

    @property
    def dark_all(self) -> bool:
        return map_from_dict(self._raw, "lightlevel", "dark_all")

    @property
    def daylight(self) -> bool:
        return map_from_dict(self._raw, "lightlevel", "daylight")

    @property
    def daylight_any(self) -> bool:
        return map_from_dict(self._raw, "lightlevel", "daylight_any")

    @property
    def lightlevel(self) -> int:
        return map_from_dict(self._raw, "lightlevel", "lightlevel")

    @property
    def lightlevel_min(self) -> int:
        return map_from_dict(self._raw, "lightlevel", "lightlevel_min")

    @property
    def lightlevel_max(self) -> int:
        return map_from_dict(self._raw, "lightlevel", "lightlevel_max")


class Group(HueObject):
    def __init__(self, identity: str, uri: str, raw: dict = None) -> None:
        self.state = GroupState("", uri, raw=raw)
        self.action = GroupAction("", uri, raw=raw)
        self.presence = GroupPresence("", uri, raw=raw)
        self.lightlevel = GroupLightLevel("", uri, raw=raw)
        super().__init__(identity, uri, raw=raw)

    def load_data(self, raw: dict = None) -> None:
        super().load_data(raw)
        self.state.load_data(self._raw)
        self.action.load_data(self._raw)
        self.presence.load_data(self._raw)
        self.lightlevel.load_data(self._raw)

    @property
    def name(self) -> str:
        return map_from_dict(self._raw, "name")

    @name.setter
    def name(self, name: str) -> None:
        self.set(name=name)

    @property
    def type(self) -> str:
        return map_from_dict(self._raw, "type")

    @property
    def lights(self) -> list[str]:
        return map_from_dict(self._raw, "lights")

    @lights.setter
    def lights(self, lights: list[str]) -> None:
        self.set(lights=lights)

    @property
    def sensors(self) -> list[str]:
        return map_from_dict(self._raw, "sensors")

    @sensors.setter
    def sensors(self, sensors: list[str]) -> None:
        self.set(sensors=sensors)

    @property
    def modelid(self) -> str:
        return map_from_dict(self._raw, "modelid")

    @property
    def uniqueid(self) -> str:
        return map_from_dict(self._raw, "uniqueid")

    @property
    def group_class(self) -> str:
        return map_from_dict(self._raw, "class")

    @group_class.setter
    def group_class(self, group_class: str) -> None:
        self.set(group_class=group_class)

    @property
    def recycle(self) -> bool:
        return map_from_dict(self._raw, "recycle")

    @recycle.setter
    def recycle(self, recycle: bool) -> None:
        self.set(recycle=recycle)

    def switch_on(self) -> None:
        self.action.on = True

    def switch_off(self) -> None:
        self.action.on = False

    def set(self, name: str = None, lights: list[str] = None, sensors: list[str] = None, group_class: str = None,
            recycle: bool = None):
        val = {}
        if name is not None:
            val["name"] = name
        if lights is not None:
            val["lights"] = lights
        if sensors is not None:
            val["sensors"] = sensors
        if group_class is not None:
            val["class"] = group_class
        if recycle is not None:
            val["recycle"] = recycle
        self.set_data("", val)
